/*----------------------------------------------------------------------
  File    : ARule.java
  Contents: association rule management for visualization
  Author  : Christian Borgelt
  History : 2004.07.06 file created
            2005.02.21 adapted to scanner utility functions
            2006.05.13 class moved to a separate file
            2007.03.12 adapted to class IdMap, javadoc added
----------------------------------------------------------------------*/
package arules;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;

import util.IdMap;
import util.Scanner;

/*--------------------------------------------------------------------*/
/** Class for association rules.
 *  @author Christian Borgelt
 *  @since  2004.07.06 */
/*--------------------------------------------------------------------*/
class ARule {

  /*------------------------------------------------------------------*/
  /*  constants                                                       */
  /*------------------------------------------------------------------*/
  /** the block size for the item vector */
  private static final int BLKSIZE = 8;

  /*------------------------------------------------------------------*/
  /*  instance variables                                              */
  /*------------------------------------------------------------------*/
  /** the underlying item set */
  protected IdMap  iset;
  /** the consequent of the association rule */
  protected int    head;
  /** the antecedent of the association rule */
  protected int[]  body;
  /** the number of items in the antecedent */
  protected int    cnt;
  /** the relative support of the body of the association rule */
  protected double b_rel;
  /** the absolute support of the body of the association rule */
  protected int    b_abs;
  /** the relative support of the item set of the association rule */
  protected double s_rel;
  /** the absolute support of the item set of the association rule */
  protected int    s_abs;
  /** the confidence of the association rule */
  protected double conf;
  /** the lift value of the association rule */
  protected double lift;
  /** the value of the additional rule evaluation measure */
  protected double arem;

  /*------------------------------------------------------------------*/
  /** Create an empty association rule.
   *  @since  2007.03.12 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public ARule ()
  { this(null); }

  /*------------------------------------------------------------------*/
  /** Create an empty association rule.
   *  @param  iset the underlying item set
   *  @since  2007.03.12 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public ARule (IdMap iset)
  {                             /* --- create an association rule */
    this.iset  = (iset != null) ? iset : new IdMap();
    this.head  = -1;            /* note the item set and */
    this.body  = null;          /* initialize the variables */
    this.s_abs = this.b_abs = this.cnt  = 0;
    this.s_rel = this.b_rel = this.conf = this.lift = this.arem = 0.0;
  }  /* ARule() */

  /*------------------------------------------------------------------*/
  /** Get the name of the consequent item.
   *  @return the name of the consequent item
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public String getHead ()
  { return (this.head >= 0) ? (String)this.iset.get(this.head) : ""; }

  /*------------------------------------------------------------------*/
  /** Get the name of an antecedent item.
   *  @param  i the index of the antecedent item
   *  @return the name of the antecedent item with index i
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public String getBody (int i)
  { return (i < this.cnt) ? (String)this.iset.get(this.body[i]) : ""; }

  /*------------------------------------------------------------------*/
  /** Get the size of the rule (total number of items).
   *  @return the size of the rule
   *  @since  2007.03.12 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int getSize ()
  { return this.cnt +((this.head >= 0) ? 1 : 0); }

  /*------------------------------------------------------------------*/
  /** Get the size of the antecedent (number of items).
   *  @return the size of the antecedent
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int getBodySize ()
  { return this.cnt; }

  /*------------------------------------------------------------------*/
  /** Get the (relative) support of the body of the association rule.
   *  @return the (relative) support of the body of the association rule
   *  @since  2008.11.03 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getBodySupp ()
  { return this.b_rel; }

  /*------------------------------------------------------------------*/
  /** Get the (absolute) support of the body of the association rule.
   *  @return the (absolute) support of the body of the association rule
   *  @since 2008.11.03 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int getAbsBodySupp ()
  { return this.b_abs; }

  /*------------------------------------------------------------------*/
  /** Get the (relative) support of the item set
   *  of the association rule.
   *  @return the (relative) support of the item set
   *          of the association rule
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getSupp ()
  { return this.s_rel; }

  /*------------------------------------------------------------------*/
  /** Get the (absolute) support of the item set
   *  of the association rule.
   *  @return the (absolute) support of the item set
   *          of the association rule
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int getAbsSupp ()
  { return this.s_abs; }

  /*------------------------------------------------------------------*/
  /** Get the confidence of the association rule.
   *  @return the confidence of the association rule
   *  @since 2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getConf ()
  { return this.conf; }

  /*------------------------------------------------------------------*/
  /** Get the lift value of the association rule.
   *  @return the lift value of the association rule
   *  @since 2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getLift ()
  { return this.lift; }

  /*------------------------------------------------------------------*/
  /** Get the value of the additional rule evaluation measure.
   *  @return the value of the additional rule evaluation measure
   *  @since 2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getAREM ()
  { return this.arem; }

  /*------------------------------------------------------------------*/
  /** Pack an association rule, i.e., optimize memory usage.
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void pack ()
  {                             /* --- pack an association rule */
    int[] vec;                  /* buffer for reallocation */
    int   vsz;                  /* size of the body vector */

    vsz = (this.body != null) ? this.body.length : 0;
    if (vsz <= this.cnt) return;
    vec = new int[vsz];         /* if the body vector is too large */
    System.arraycopy(this.body, 0, vec, 0, this.cnt);
    this.body = vec;            /* shrink the body vector */
  }  /* pack() */

  /*------------------------------------------------------------------*/
  /** Create a 'malformed number' I/O exception.
   *  @param  scan the scanner to read from
   *  @return a 'malformed number' I/O exception.
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  private static IOException badNumEx (Scanner scan)
  { return new IOException("malformed number '" +scan.value +"'"
                           +scan.lno()); }

  /*------------------------------------------------------------------*/
  /** Parse an association rule.
   *  @param  iset the underlying item set
   *  @param  scan the scanner to read from
   *  @return the parsed association rule
   *  @throws IOException if a read error occurs
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static ARule parseARule (IdMap iset, Scanner scan)
    throws IOException
  {                             /* --- parse an association rule */
    ARule rule;                 /* created association rule */
    int[] vec;                  /* buffer for reallocation */
    int   vsz;                  /* size of the antecedent vector */

    rule = new ARule(iset);     /* create an association rule */
    scan.getID();               /* get the consequent item */
    rule.head = iset.add(scan.value);
    if (scan.nextToken() != Scanner.T_LFT)
      throw new IOException("'<-' expected " +scan.lno());
    rule.body = new int[vsz = BLKSIZE];
    while ((scan.nextToken() == Scanner.T_ID)
    ||     (scan.ttype       == Scanner.T_NUM)) {
      if (rule.cnt >= vsz) {    /* if the antecedent vector is full */
        vsz += (vsz > BLKSIZE) ? vsz >> 1 : BLKSIZE;
        vec = new int[vsz];     /* create a new antecedent vector */
        System.arraycopy(rule.body, 0, vec, 0, rule.cnt);
        rule.body = vec;        /* copy the vector contents */
      }                         /* and set the new vector */
      rule.body[rule.cnt++] = rule.iset.add(scan.value);
    }                           /* store the antecedent items */
    scan.pushBack();            /* unget last token */
    scan.getChar('(');          /* check for a '(' */
    scan.getNumber();           /* get the relative body support */
    try { rule.b_rel = Double.parseDouble(scan.value); }
    catch (NumberFormatException e) { throw ARule.badNumEx(scan); }
    scan.getChar('/');          /* check for a '/' */
    scan.getNumber();           /* get the absolute body support */
    try { rule.b_abs = Integer.parseInt(scan.value); }
    catch (NumberFormatException e) { throw ARule.badNumEx(scan); }
    scan.getChar(',');          /* check for a ',' */
    scan.getNumber();           /* get the relative support */
    try { rule.s_rel = Double.parseDouble(scan.value); }
    catch (NumberFormatException e) { throw ARule.badNumEx(scan); }
    scan.getChar('/');          /* check for a '/' */
    scan.getNumber();           /* get the absolute support */
    try { rule.s_abs = Integer.parseInt(scan.value); }
    catch (NumberFormatException e) { throw ARule.badNumEx(scan); }
    scan.getChar(',');          /* check for a ',' */
    scan.getNumber();           /* get the confidence */
    try { rule.conf = Double.parseDouble(scan.value); }
    catch (NumberFormatException e) { throw ARule.badNumEx(scan); }
    scan.getChar(',');          /* check for a ',' */
    scan.getNumber();           /* get the lift value */
    try { rule.lift = Double.parseDouble(scan.value); }
    catch (NumberFormatException e) { throw ARule.badNumEx(scan); }
    if (scan.nextToken() != ',') { /* if no further number follows */
      rule.arem = -Double.MAX_VALUE;
      scan.pushBack(); }        /* invalidate the add. evaluation */
    else {                      /* otherwise consume the number */
      scan.getNumber();         /* get the additional evaluation */
      try { rule.arem = Double.parseDouble(scan.value); }
      catch (NumberFormatException e) { throw ARule.badNumEx(scan); }
    }
    scan.getChar(')');          /* check for a ')' */
    rule.pack();                /* pack the parsed association rule */
    return rule;                /* and then return it */
  }  /* parseARule() */

  /*------------------------------------------------------------------*/
  /** Parse an association rule.
   *  @param  iset   the underlying item set
   *  @param  reader the reader to read from
   *  @return the parsed association rule
   *  @throws IOException if a read error occurs
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static ARule parseARule (IdMap iset, Reader reader)
    throws IOException
  { return ARule.parseARule(iset, new Scanner(reader)); }

  /*------------------------------------------------------------------*/
  /** Parse an association rule.
   *  @param  iset the underlying item set
   *  @param  desc the string description to parse
   *  @return the parsed association rule
   *  @throws IOException if a read error occurs
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static ARule parseARule (IdMap iset, String desc)
    throws IOException
  { return ARule.parseARule(iset, new Scanner(desc)); }

  /*------------------------------------------------------------------*/
  /** Parse an association rule.
   *  @param  iset the underlying item set
   *  @param  in   the input stream to read from
   *  @return the parsed association rule
   *  @throws IOException if a read error occurs
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static ARule parseARule (IdMap iset, InputStream in)
    throws IOException
  { return ARule.parseARule(iset, new Scanner(in)); }

  /*------------------------------------------------------------------*/
  /** Create a string description.
   *  @return the created string description
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public String toString ()
  {                             /* --- create a string representation */
    int          i;             /* loop variable */
    StringBuffer b;             /* created string representation */
    
    b = new StringBuffer();     /* start with the consequent */
    if (this.head >= 0) b.append(this.iset.get(this.head));
    b.append(" <-");            /* add implication sign */
    for (i = 0; i < this.cnt; i++) {
      b.append(" "); b.append(this.iset.get(this.body[i])); }
    b.append("  (");            /* add the antecedent items */
    b.append(this.s_rel); b.append("/");    /* add evaluation */
    b.append(this.s_abs); b.append(", ");   /* information */
    b.append(this.b_rel); b.append("/");
    b.append(this.b_abs); b.append(", ");
    b.append(this.conf);  b.append(", ");
    b.append(this.lift);  b.append(", ");
    b.append(this.arem);  b.append(")");
    return b.toString();        /* return the string representation */
  }  /* toString() */

}  /* class ARule */
