/*----------------------------------------------------------------------
  File    : ARuleGUI.java
  Contents: association rule induction toolbox
  Author  : Christian Borgelt
  History : 2004.09.29 file created from file DTreeGUI.java
            2007.05.08 adapted to new executor classes
            2007.05.30 mincnt and maxcnt made JSpinners
            2007.06.05 number inputs turned into JFormattedTextFields
            2007.07.07 adapted to new class DialogPanel
----------------------------------------------------------------------*/
package arules;

import java.io.IOException;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import javax.swing.JTextField;
import javax.swing.JSpinner;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;

import dialog.TabbedGUI;
import dialog.DialogPanel;
import dialog.AboutPanel;
import dialog.FormatDialog;
import util.Executor;
import util.CmdExecutor;

/*--------------------------------------------------------------------*/
/** Class for a user interface to the apriori program.
 *  @author Christian Borgelt
 *  @since  2004.09.29 */
/*--------------------------------------------------------------------*/
public class ARuleGUI extends TabbedGUI implements ActionListener {

  /*------------------------------------------------------------------*/
  /*  constants                                                       */
  /*------------------------------------------------------------------*/
  private static final long serialVersionUID = 1L;
  /** the names of the additional rule evaluation measures */
  private static final String[] emnames = {
    "none",
    "rule confidence",
    "abs. confidence difference to prior",
    "lift value (confidence / prior)",
    "abs. difference of lift value to 1",
    "difference of lift quotient to 1",
    "normalized chi^2 measure",
    "p-value computed from chi^2 measure",
    "information difference to prior",
    "p-value computed from G statistic" };

  /** the codes of the additional rule evaluation measures */
  private static final char[] emcodes = {
    'x', 'c', 'd', 'l', 'a', 'r', 'n', 'p', 'i', 'g' };

  /** the names of the transaction sorting modes */
  private static final String[] sortnames = {
    "descending w.r.t. transaction size sum",
    "descending w.r.t. frequency",
    "no sorting",
    "ascending w.r.t. frequency",
    "ascending w.r.t. transaction size sum" };

  /*------------------------------------------------------------------*/
  /*  instance variables                                              */
  /*------------------------------------------------------------------*/
  /* --- files --- */
  /** the transactions file */
  private JTextField fn_tra;
  /** the association rule file */
  private JTextField fn_rul;
  /** the item appearances file */
  private JTextField fn_app;
  /** the path to the C programs */
  private File       path;

  /* --- filters --- */
  /** the minimum number of items per rule */
  private JSpinner   mincnt;
  /** the maximum number of items per rule */
  private JSpinner   maxcnt;
  /** the minimum support of a rule */
  private JTextField minsupp;
  /** the maximum support of a rule */
  private JTextField maxsupp;
  /** whether to use the original support definition */
  private JCheckBox  orig;
  /** the minimum confidence of a rule */
  private JTextField minconf;
  /** the additional rule evaluation measure */
  private JComboBox  arem;
  /** the minimum value of the additional rule evaluation measure */
  private JTextField minarem;

  /* --- options --- */
  /** the sorting mode for the transactions */
  private JComboBox  sort;
  /** whether to organize the transactions as a prefix tree */
  private JCheckBox  prefix;
  /** whether to use quicksort */
  private JCheckBox  quick;
  /** the output format for the rule information */
  private JTextField outfmt;

  /*------------------------------------------------------------------*/
  /** Create an association rule GUI.
   *  <p>The dialog is created as a stand-alone program.
   *  That is, closing or quitting it terminates the program.</p>
   *  @since  2004.09.29 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public ARuleGUI ()
  { this.init(null, true); }

  /*------------------------------------------------------------------*/
  /** Create an association rule GUI.
   *  @param  owner the component that is to own this dialog
   *  @since  2004.09.29 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public ARuleGUI (Component owner)
  { this.init(owner, false); }

  /*------------------------------------------------------------------*/
  /** Create the user interface.
   *  <p>Following the recommendations in the Java tutorial, the user
   *  interface is created in the "run" method, which is invoked from
   *  the event queue, in order to avoid problems with threads.</p>
   *  @since  2007.03.12 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void run ()
  {                             /* --- create the user interface */
    DialogPanel tab;            /* current tab of the tabbed pane */

    /* --- create basic GUI --- */
    this.base("Association Rule Induction Tools");
    this.addFormatTab(0);

    /* --- files --- */
    tab = this.addTab("Files");

    tab.addLabel("Transactions file:");
    tab.addButton("Select", DialogPanel.MIDDLE).addActionListener(
      new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        ARuleGUI.this.getFileName(ARuleGUI.this.fn_tra); } } );
    tab.addButton("View", DialogPanel.RIGHT).addActionListener(
      new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        ARuleGUI.this.showTrActs(ARuleGUI.this.fn_tra); } } );
    this.fn_tra = tab.addFileInput("noname.tra");

    tab.addLabel("Association rules file:");
    tab.addButton("Select", DialogPanel.MIDDLE).addActionListener(
      new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        ARuleGUI.this.getFileName(ARuleGUI.this.fn_rul); } } );
    tab.addButton("View", DialogPanel.RIGHT).addActionListener(
      new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        ARuleGUI.this.showRules(ARuleGUI.this.fn_rul); } } );
    this.fn_rul = tab.addFileInput("noname.rul");

    tab.addLabel("Item appearances file:");
    tab.addButton("Select", DialogPanel.MIDDLE).addActionListener(
      new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        ARuleGUI.this.getFileName(ARuleGUI.this.fn_app); } } );
    tab.addButton("Edit", DialogPanel.RIGHT).addActionListener(
      new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        ARuleGUI.this.editFile(ARuleGUI.this.fn_app); } } );
    this.fn_app = tab.addFileInput("");

    tab.addHelp("The item appearances file is optional.\n"
               +"It specifies in which parts of an association rule\n"
               +"(antecedent/consequent) an item "
               +"is allowed to appear.");
    tab.addFiller(0);

    tab.addButton("Locate Program...", DialogPanel.RIGHT)
      .addActionListener(new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        JFileChooser c = ARuleGUI.this.getFileChooser();
        c.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        c.setDialogTitle("Locate Program...");
        int r = c.showOpenDialog(ARuleGUI.this);
        if (r == JFileChooser.APPROVE_OPTION)
          ARuleGUI.this.path = c.getSelectedFile();
        c.setFileSelectionMode(JFileChooser.FILES_ONLY);
        c.setDialogTitle("Open File...");
      } } );

    /* --- filters --- */
    tab = this.addTab("Filters");

    tab.addLabel("Minimal number of items:");
    this.mincnt = tab.addSpinner(1, 1, 999999, 1);
    tab.addLabel("Maximal number of items:");
    this.maxcnt = tab.addSpinner(0, 0, 999999, 1);

    tab.addLabel("Minimal support [%]:");
    this.minsupp = tab.addNumberInput("10");
    tab.addLabel("Maximal support [%]:");
    this.maxsupp = tab.addNumberInput("100");
    tab.addLabel("Minimal confidence [%]:");
    this.minconf = tab.addNumberInput("80");

    tab.addLabel("Original support definition:");
    this.orig = tab.addCheckBox(false);
    tab.addHelp("With the original support definition the support of\n"
               +"an association rule is the support of all occurring "
               +"items.\nBy default it is only the support "
               +"of the antecendent.");
    tab.addFiller(4);

    tab.addLabel("Additional evaluation measure:", DialogPanel.RIGHT);
    this.arem = tab.addComboBox(ARuleGUI.emnames);
    tab.addLabel("Minimal value [%]:");
    this.minarem = tab.addNumberInput("10");
    tab.addFiller(0);

    /* --- options --- */
    tab = this.addTab("Options");

    tab.addLabel("Item sorting:", DialogPanel.RIGHT);
    this.sort = tab.addComboBox(ARuleGUI.sortnames);
    this.sort.setSelectedIndex(4);
    tab.addHelp("Using a different item sorting than the default\n"
               +"usually slows down processing.");

    tab.addLabel("Transactions as prefix tree:");
    this.prefix = tab.addCheckBox(true);
    tab.addHelp("Representing the transactions as a prefix tree\n"
               +"usually speeds up processing.");

    tab.addLabel("Use quicksort:");
    this.quick = tab.addCheckBox(false);
    tab.addHelp("The default is to sort the transactions "
               +"with heapsort,\nwhich can be slightly slower, "
               +"but is more robust.");

    tab.addLabel("Information output format:", DialogPanel.FILL);
    this.outfmt = tab.addTextInput(" (%1X/%b, %1S/%a, %1C, %3l, %1E)",
                                   DialogPanel.FILL);
    tab.addHelp("The information output format is very important\n"
               +"for correct parsing of the found association rules.\n"
               +"Change at most the number of decimal places\n"
               +"and S, X, C or E to lowercase letters.");
    tab.addFiller(0);

    /* --- about --- */
    this.addTab("About", new AboutPanel(
       "Association Rule Induction Tools",
       "A simple user interface for the apriori program.\n\n"
      +"Version 2.0, 2008.11.03\n"
      +"written by Christian Borgelt\n"
      +"European Center for Soft Computing\n"
      +"c/ Gonzalo Gutierrez Quiros s/n\n"
      +"33600 Mieres, Asturias, Spain\n"
      +"christian.borgelt@softcomputing.es"));

    /* --- finalize --- */
    this.pack();
    this.selectTab(1);
  }  /* run() */

  /*------------------------------------------------------------------*/
  /** Get the path to the apriori program.
   *  @return the path to the apriori program
   *  @since  2007.03.12 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public File getPath ()
  { return this.path; }

  /*------------------------------------------------------------------*/
  /** Set the path to the apriori program.
   *  @param  path the path to the apriori program
   *  @since  2007.03.12 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void setPath (File path)
  { this.path = path; }

  /*------------------------------------------------------------------*/
  /** Show association rules.
   *  @param  txt the text field containing the file name
   *  @since  2007.03.12 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  protected void showRules (JTextField txt)
  { this.showRules(new File(txt.getText())); }

  /*------------------------------------------------------------------*/
  /** Show a set of association rules.
   *  @param  file the file to load the association rules from
   *  @since  2007.03.12 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  protected void showRules (File file)
  {                             /* --- show a set of ass. rules */
    ARuleView view = new ARuleView(this, ARuleView.FILE_ITEMS);
    if (!view.loadRules(file)) return;
    view.setVisible(true); view.toFront();
  }  /* showRules() */

  /*------------------------------------------------------------------*/
  /** Show a set of transactions.
   *  @param  txt the text field containing the file name
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  protected void showTrActs (JTextField txt)
  { this.showTrActs(new File(txt.getText())); }

  /*------------------------------------------------------------------*/
  /** Show a set of transactions.
   *  @param  file the file to load the transactions from
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  protected void showTrActs (File file)
  {                             /* --- show a set of ass. rules */
    TrActView    view;          /* viewer for the transactions */
    FormatDialog fmtdlg;        /* format dialog of the viewer */

    view = new TrActView(this, TrActView.FILE_ITEMS);
    if (this.format != null) {  /* if there is a format tab */
      fmtdlg = view.getFormatDlg();
      fmtdlg.setRecSeps  (this.format.getRecSeps());
      fmtdlg.setFldSeps  (this.format.getFldSeps());
      fmtdlg.setBlanks   (this.format.getBlanks());
      fmtdlg.setComment  (this.format.getComment());
    }                           /* transfer characters */
    if (!view.loadTrActs(file)) return;
    view.setVisible(true); view.toFront();
  }  /* showtrActs() */

  /*------------------------------------------------------------------*/
  /** Get an executor for the apriori program.
   *  @return the executor for the apriori program
   *  @since  2007.03.12 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  protected Executor createExecutor (int i)
  {                             /* --- get the command to execute */
    int      n = 0, k;          /* number of arguments, buffer */
    String[] cmd;               /* command and arguments */
    File     pp;                /* the path to the programs */
    String   s;                 /* buffer for arguments */

    cmd    = new String[32];    /* create a command array */
    pp     = this.getPath();    /* and get the program path */
    cmd[0] = ((pp != null) ? pp +"/" : "") +"apriori";
    n = this.format.addFormatArgs(cmd);
    cmd[n++] = "-m" +this.mincnt.getValue();
    k = ((Integer)this.maxcnt.getValue()).intValue();
    if (k > 0) cmd[n++] = "-n" +k;
    if (this.orig.isSelected()) cmd[n++] = "-o";
    s = this.minsupp.getText(); /* minimum support */
    if (s.length() > 0) cmd[n++] = "-s" +s;
    s = this.maxsupp.getText(); /* maximum support */
    if (s.length() > 0) cmd[n++] = "-S" +s;
    s = this.minconf.getText(); /* minimum confidence */
    if (s.length() > 0) cmd[n++] = "-c" +s;
    cmd[n++] = "-gtr";          /* scanable form for items */
    s = this.outfmt.getText();  /* output format */
    if (s.length() > 0) cmd[n++] = "-v" +s;
    k = this.arem.getSelectedIndex();
    if (k > 0) {                /* additional rule evaluation measure */
      cmd[n++] = "-e" +emcodes[k]; cmd[n++] = "-d" +s; }
    k = this.sort.getSelectedIndex();
    if (k != 2) cmd[n++] = "-q" +(k-2);
    if (this.prefix.isSelected()) cmd[n++] = "-h";
    if (this.quick.isSelected())  cmd[n++] = "-j";
    cmd[n++] = fn_tra.getText();
    cmd[n++] = fn_rul.getText();
    s = fn_app.getText();       /* add the file arguments */
    if (s.length() > 0) cmd[n++] = s;
    return new CmdExecutor(TabbedGUI.shrinkCmd(cmd, n), this);
  }  /* createExecutor() */

  /*------------------------------------------------------------------*/
  /** Get the result message on successful termination.
   *  @return the result message
   *  @since  2007.02.10 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  protected String getResultMsg ()
  {                             /* --- get result message */
    int    i, k;                /* index variables */
    String msg;                 /* buffer for message */
    float  t;                   /* execution time */

    msg = ((CmdExecutor)this.executor).getLastErrorLine();
    i = msg.lastIndexOf("[", msg.indexOf("rule(s)]"));
    k = msg.indexOf(']', i);    /* find rules report */
    t = (System.currentTimeMillis() -this.start) / 1000.0F;
    return "Found " +msg.substring(i+1, k) +".\n"
          +"Total search time: " +t +"s.\n"
          +"(See terminal for more information.)";
  }  /* getResultMessage() */

  /*------------------------------------------------------------------*/
  /** Load a configuration file and set the input fields.
   *  @param  file the file to load the configuration from
   *  @since  2007.07.08 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  protected void loadConfig (File file)
  {                             /* --- load configuration file */
    if (file == null) {         /* if no file name is given */
      JFileChooser c = this.getFileChooser();
      c.setDialogTitle("Load Configuration...");
      int r = c.showOpenDialog(this);
      if (r != JFileChooser.APPROVE_OPTION) return;
      file = c.getSelectedFile();
    }                           /* let the user choose a file */
    try {                       /* open the configuration file */
      FileReader reader = new FileReader(file);
      this.fn_tra.setText (this.readLine(reader));
      this.fn_rul.setText (this.readLine(reader));
      this.fn_app.setText (this.readLine(reader));
      this.minsupp.setText(this.readLine(reader));
      this.maxsupp.setText(this.readLine(reader));
      this.minconf.setText(this.readLine(reader));
      this.minarem.setText(this.readLine(reader));
      this.outfmt.setText (this.readLine(reader));
      this.mincnt.setValue(new Integer(this.readInt(reader)));
      this.maxcnt.setValue(new Integer(this.readInt(reader)));
      this.arem.setSelectedIndex(this.readInt(reader));
      this.sort.setSelectedIndex(this.readInt(reader));
      this.orig.setSelected     (this.readInt(reader) != 0);
      this.prefix.setSelected   (this.readInt(reader) != 0);
      this.quick.setSelected    (this.readInt(reader) != 0);
      reader.close(); }         /* read the configuration values */
    catch (IOException e) {     /* and close the input file */
      JOptionPane.showMessageDialog(this,
        "Error reading configuration file:\n" +e.getMessage(),
        "Error", JOptionPane.ERROR_MESSAGE);
    }                           /* check for successful reading */
    this.status.setText("configuration loaded: " +file.getName());
  }  /* loadConfig() */

  /*------------------------------------------------------------------*/
  /** Save a configuration file
   *  @param  file the file to save the current configuration to
   *  @since  2007.07.08 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  protected void saveConfig (File file)
  {                             /* --- save configuration file */
    if (file == null) {         /* if no file name is given, */
      JFileChooser c = this.getFileChooser();
      c.setDialogTitle("Save Configuration...");
      int r = c.showOpenDialog(this);
      if (r != JFileChooser.APPROVE_OPTION) return;
      file = c.getSelectedFile();
    }                           /* let the user choose a file */
    try {                       /* open the configuration file */
      FileWriter writer = new FileWriter(file);
      writer.write(this.fn_tra.getText());  writer.write('\n');
      writer.write(this.fn_rul.getText());  writer.write('\n');
      writer.write(this.fn_app.getText());  writer.write('\n');
      writer.write(this.minsupp.getText()); writer.write('\n');
      writer.write(this.maxsupp.getText()); writer.write('\n');
      writer.write(this.minconf.getText()); writer.write('\n');
      writer.write(this.minarem.getText()); writer.write('\n');
      writer.write(this.outfmt.getText());  writer.write('\n');
      writer.write(((Integer)this.mincnt.getValue()).intValue() +",");
      writer.write(((Integer)this.maxcnt.getValue()).intValue() +",");
      writer.write(this.arem.getSelectedIndex() +",");
      writer.write(this.sort.getSelectedIndex() +",");
      writer.write(this.orig.isSelected()   ? "1," : "0,");
      writer.write(this.prefix.isSelected() ? "1," : "0,");
      writer.write(this.quick.isSelected()  ? "1," : "0,");
      writer.write('\n');       /* write the configuration values */
      writer.close(); }         /* and close the output file */
    catch (IOException e) {     /* check for successful writing */
      JOptionPane.showMessageDialog(this,
        "Error writing configuration file:\n" +e.getMessage(),
        "Error", JOptionPane.ERROR_MESSAGE);
    }                           /* show a status message */
    this.status.setText("configuration saved: " +file.getName());
  }  /* saveConfig() */

  /*------------------------------------------------------------------*/
  /** Main function for command line invocation.
   *  @param  args the command line arguments
   *  @since  2004.09.29 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static void main (String args[])
  {                             /* --- main function */
    ARuleGUI gui = new ARuleGUI();
    if (args.length > 0)        /* load configuration if necessary */
      gui.loadConfig(new File(args[0]));
    gui.setVisible(true);       /* show the user interface */
  }  /* main() */

}  /* class ARuleGUI */
