/*----------------------------------------------------------------------
  File    : ARuleSet.java
  Contents: association rule management for visualization
  Author  : Christian Borgelt
  History : 2004.07.06 file created
            2004.12.09 adapted to changed additional evaluation output
            2005.01.24 incomplete adaptation completed
            2005.02.21 adapted to scanner utility functions
            2007.03.12 adapted to class IdMap, javadoc added
            2010.01.25 access to comparators modified in sort()
----------------------------------------------------------------------*/
package arules;

import java.io.IOException;
import java.io.Reader;
import java.io.FileReader;
import java.io.Writer;
import java.io.PrintWriter;
import java.util.Comparator;
import java.util.Arrays;

import util.IdMap;
import util.Scanner;

/*--------------------------------------------------------------------*/
/** Class for a set of association rules.
 *  @author Christian Borgelt
 *  @since  2004.07.06 */
/*--------------------------------------------------------------------*/
public class ARuleSet {

  private static final long serialVersionUID = 1L;

  /*------------------------------------------------------------------*/
  /*  constants                                                       */
  /*------------------------------------------------------------------*/
  /** the block size for the rule vector */
  private static final int BLKSIZE = 64;
  /** sort mode: size / number of items */
  public  static final int SIZE    = 0;
  /** sort mode: confidence */
  public  static final int CONF    = 1;
  /** sort mode: support */
  public  static final int SUPP    = 2;
  /** sort mode: body support */
  public  static final int BODY    = 3;
  /** sort mode: lift value */
  public  static final int LIFT    = 4;
  /** sort mode: additional rule evaluation measure */
  public  static final int AREM    = 5;
  /** sort mode: head of the rule */
  public  static final int HEAD    = 6;

  /*------------------------------------------------------------------*/
  /*  instance variables                                              */
  /*------------------------------------------------------------------*/
  /** the underlying item set */
  protected IdMap   iset;
  /** the set of association rules */
  protected ARule[] rules;
  /** the current number of rules */
  protected int     cnt;
  /** the maximal number of items in a rule body */
  protected int     body;
  /** the maximal number of items in a rule head */
  protected int     head;

  /*------------------------------------------------------------------*/
  /** Create an empty set of association rules.
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public ARuleSet ()
  { this(null); }

  /*------------------------------------------------------------------*/
  /** Create an empty set of association rules.
   *  @param  iset the underlying item set
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public ARuleSet (IdMap iset)
  {                             /* --- create an association rule set */
    this.iset  = (iset != null) ? iset : new IdMap();
    this.rules = new ARule[BLKSIZE];
    this.cnt   = this.body = this.head = 0;
  }  /* ARuleSet() */

  /*------------------------------------------------------------------*/
  /** Get the underlying item set.
   *  @return the underlying item set
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public IdMap getItemSet ()
  { return this.iset; }

  /*------------------------------------------------------------------*/
  /** Get the number of association rules.
   *  @return the number of association rules
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int getRuleCount ()
  { return this.cnt; }

  /*------------------------------------------------------------------*/
  /** Get an association rule
   *  @param  i the index of the association rule
   *  @return the association rule with index i
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public ARule getRule (int i)
  { return this.rules[i]; }

  /*------------------------------------------------------------------*/
  /** Add an association rule.
   *  @param  rule the association rule to add
   *  @return the index of the association rule in the set
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int addRule (ARule rule)
  {                             /* --- add an association rule */
    ARule[] vec;                /* buffer for reallocation */
    int     vsz;                /* new vector size */

    vsz = this.rules.length;    /* get the size of the rule vector */
    if (this.cnt >= vsz) {      /* if the rule vector is full */
      vsz += (vsz > BLKSIZE) ? vsz >> 1 : BLKSIZE;
      vec = new ARule[vsz];     /* create a new rule vector */
      System.arraycopy(this.rules, 0, vec, 0, this.cnt);
      this.rules = vec;         /* copy the vector contents */
    }                           /* and set the new vector */
    this.rules[this.cnt] = rule;/* add the new rule to the set */
    if (rule.cnt > this.body) this.body = rule.cnt;
    if (rule.head >= 0)       this.head = 1;
    return this.cnt++;          /* adapt the maximal sizes and */
  }  /* addRule() */            /* return the rule identifier */

  /*------------------------------------------------------------------*/
  /** Get the maximal size of a rule.
   *  @return the maximal size of a rule
   *  @since  2007.03.12 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int getMaxSize ()
  { return this.body +this.head; }

  /*------------------------------------------------------------------*/
  /** Get the maximal size of a rule body.
   *  @return the maximal size of a rule body
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int getMaxBodySize ()
  { return this.body; }

  /*------------------------------------------------------------------*/
  /** Sort the set of association rules.
   *  @param  field the identifier of the field to compare first
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void sort (int field)
  {                             /* --- sort the rule set */
    Comparator<Object> cmp;     /* comparator for sorting the rules */

    if      (field == SIZE)
      // #if generics
      cmp = new Comparator<Object> () {
      // #else
      //`cmp = new Comparator () {
      // #endif
        public int compare (Object o1, Object o2) {
          int n1 = ((ARule)o1).getSize();
          int n2 = ((ARule)o2).getSize();
          if (n1 > n2) return +1;
          if (n1 < n2) return -1;
          return 0; } };
    else if (field == CONF)
      // #if generics
      cmp = new Comparator<Object> () {
      // #else
      //`cmp = new Comparator () {
      // #endif
        public int compare (Object o1, Object o2) {
          if (((ARule)o1).conf  < ((ARule)o2).conf) return +1;
          if (((ARule)o1).conf  > ((ARule)o2).conf) return -1;
          return 0; } };
    else if (field == SUPP)
      // #if generics
      cmp = new Comparator<Object> () {
      // #else
      //`cmp = new Comparator () {
      // #endif
        public int compare (Object o1, Object o2) {
          if (((ARule)o1).s_rel < ((ARule)o2).s_rel) return +1;
          if (((ARule)o1).s_rel > ((ARule)o2).s_rel) return -1;
          if (((ARule)o1).s_abs < ((ARule)o2).s_abs) return +1;
          if (((ARule)o1).s_abs > ((ARule)o2).s_abs) return -1;
          return 0; } };
    else if (field == BODY)
      // #if generics
      cmp = new Comparator<Object> () {
      // #else
      //`cmp = new Comparator () {
      // #endif
        public int compare (Object o1, Object o2) {
          if (((ARule)o1).b_rel < ((ARule)o2).b_rel) return +1;
          if (((ARule)o1).b_rel > ((ARule)o2).b_rel) return -1;
          if (((ARule)o1).b_abs < ((ARule)o2).b_abs) return +1;
          if (((ARule)o1).b_abs > ((ARule)o2).b_abs) return -1;
          return 0; } };
    else if (field == LIFT)
      // #if generics
      cmp = new Comparator<Object> () {
      // #else
      //`cmp = new Comparator () {
      // #endif
        public int compare (Object o1, Object o2) {
          if (((ARule)o1).lift  < ((ARule)o2).lift) return +1;
          if (((ARule)o1).lift  > ((ARule)o2).lift) return -1;
          return 0; } };
    else if (field == AREM)
      // #if generics
      cmp = new Comparator<Object> () {
      // #else
      //`cmp = new Comparator () {
      // #endif
        public int compare (Object o1, Object o2) {
          if (((ARule)o1).arem  < ((ARule)o2).arem) return +1;
          if (((ARule)o1).arem  > ((ARule)o2).arem) return -1;
          return 0; } };
    else if (field == HEAD)
      // #if generics
      cmp = new Comparator<Object> () {
      // #else
      //`cmp = new Comparator () {
      // #endif
        public int compare (Object o1, Object o2) {
          if (((ARule)o1).head < 0)
            return (((ARule)o2).head < 0) ? 0 : -1;
          if (((ARule)o2).head < 0) return +1;
          return ((ARule)o1).getHead().compareTo(((ARule)o2).getHead());
        } };
    else return;
    Arrays.sort(this.rules, 0, this.cnt, cmp);
  }  /* sort() */

  /*------------------------------------------------------------------*/
  /** Write a set of association rules.
   *  @param  writer the writer to write to
   *  @throws IOException if a write error occurs
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void write (Writer writer) throws IOException
  {                             /* --- write a set of ass. rules */
    for (int i = 0; i < this.cnt; i++)
      writer.write(this.rules[i].toString());
  }  /* write() */

  /*------------------------------------------------------------------*/
  /** Parse a set of association rules.
   *  @param  iset the underlying item set
   *  @param  scan the scanner to read from
   *  @return the parsed set of association rules
   *  @throws IOException if a read error occurs
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static ARuleSet parseARuleSet (IdMap iset, Scanner scan)
    throws IOException
  {                             /* --- parse an association rule set */
    ARuleSet arset = new ARuleSet(iset);
    while (scan.nextToken() != Scanner.T_EOF) {
      scan.pushBack();          /* while not at end of input */
      arset.addRule(ARule.parseARule(arset.iset, scan));
    }                           /* parse the association rules */
    return arset;               /* and return the created set */
  }  /* parseARuleSet() */

  /*------------------------------------------------------------------*/
  /** Parse a set of association rules.
   *  @param  iset   the underlying item set
   *  @param  reader the reader to read from
   *  @return the parsed set of association rules
   *  @throws IOException if a read error occurs
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static ARuleSet parseARuleSet (IdMap iset, Reader reader)
    throws IOException
  { return ARuleSet.parseARuleSet(iset, new Scanner(reader)); }

  /*------------------------------------------------------------------*/
  /** Main function for testing some basic functionality.
   *  <p>It is tried to parse the file that is given as the first
   *  command line argument as a set of association rules.</p>
   *  @param  args the command line arguments
   *  @since  2005.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static void main (String args[])
  {                             /* --- main function for testing */
    Scanner  scan;              /* scanner to read from */
    ARuleSet arset;             /* created association rule set */

    try {                       /* try to read the given file */
      if (args.length <= 0)     /* if no arguments given */
        scan = new Scanner("a <- b c (1/3, 20, 80, 150)");
      else                      /* if a file argument is given */
        scan = new Scanner(new FileReader(args[0]));
      arset = ARuleSet.parseARuleSet(null, scan);
      arset.write(new PrintWriter(System.out)); }
    catch (IOException e) {     /* parse and print the rule set */
      System.err.println(e.getMessage()); return; }
  }  /* main() */

}  /* class ARuleSet */
