/*----------------------------------------------------------------------
  File    : TrActSet.java
  Contents: transaction set management for visualization
  Author  : Christian Borgelt
  History : 2007.06.06 file created
----------------------------------------------------------------------*/
package arules;

import java.io.IOException;
import java.io.FileReader;

import util.IdMap;
import util.TableReader;
import util.TableWriter;

/*--------------------------------------------------------------------*/
/** Class for a set of transactions.
 *  @author Christian Borgelt
 *  @since  2007.06.06 */
/*--------------------------------------------------------------------*/
public class TrActSet {

  /*------------------------------------------------------------------*/
  /*  constants                                                       */
  /*------------------------------------------------------------------*/
  /** the block size for the transactions vector */
  private static final int BLKSIZE = 32;

  /*------------------------------------------------------------------*/
  /*  instance variables                                              */
  /*------------------------------------------------------------------*/
  /** the underlying item set */
  protected IdMap   iset;
  /** the set of transactions */
  protected int[][] tracts;
  /** the current number of transactions */
  protected int     cnt;
  /** the maximal size of a transaction */
  protected int     max;

  /*------------------------------------------------------------------*/
  /** Create an empty set of transactions.
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TrActSet ()
  { this(null); }

  /*------------------------------------------------------------------*/
  /** Create an empty set of transactions.
   *  @param  iset the underlying item set
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TrActSet (IdMap iset)
  {                             /* --- create a transaction set */
    this.iset   = (iset != null) ? iset : new IdMap();
    this.tracts = new int[BLKSIZE][];
    this.cnt    = this.max = 0;
  }  /* TrActSet() */

  /*------------------------------------------------------------------*/
  /** Get the underlying item set.
   *  @return the underlying item set
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public IdMap getItemSet ()
  { return this.iset; }

  /*------------------------------------------------------------------*/
  /** Get the name of an item.
   *  @return the name of the item with index <code>i</code>
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public String getItemName (int i)
  { return (String)this.iset.get(i); }

  /*------------------------------------------------------------------*/
  /** Get the number of transactions.
   *  @return the number of transactions
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int getTrActCount ()
  { return this.cnt; }

  /*------------------------------------------------------------------*/
  /** Get a transaction.
   *  @param  i the index of the transaction
   *  @return the transaction with index <code>i</code>
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int[] getTrAct (int i)
  { return this.tracts[i]; }

  /*------------------------------------------------------------------*/
  /** Add a transaction.
   *  @param  tract the transaction to add
   *  @return the index of the transaction in the set
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int addTrAct (int[] tract)
  {                             /* --- add a transaction */
    int[][] vec;                /* buffer for reallocation */
    int     vsz;                /* new vector size */

    vsz = this.tracts.length;   /* get the size of the vector */
    if (this.cnt >= vsz) {      /* if the transaction vector is full */
      vsz += (vsz > BLKSIZE) ? vsz >> 1 : BLKSIZE;
      vec = new int[vsz][];     /* create a new transaction vector */
      System.arraycopy(this.tracts, 0, vec, 0, this.cnt);
      this.tracts = vec;        /* copy the vector contents */
    }                           /* and set the new vector */
    this.tracts[this.cnt] = tract; /* add the new transaction */
    if (tract.length > this.max) this.max = tract.length;
    return this.cnt++;          /* adapt the maximal size and */
  }  /* addTrAct() */           /* return the transaction identifier */

  /*------------------------------------------------------------------*/
  /** Get the maximal size of a transaction.
   *  @return the maximal size of a transaction
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int getMaxSize ()
  { return this.max; }

  /*------------------------------------------------------------------*/
  /** Write a set of transactions.
   *  @param  writer the table writer to write to
   *  @throws IOException if a write error occurs
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void write (TableWriter writer) throws IOException
  {                             /* --- write a transaction set */
    int   i, k;                 /* loop variables */
    int[] t;                    /* to traverse the transactions */

    for (i = 0; i < this.cnt; i++) {
      t = this.tracts[i];       /* traverse the transactions */
      for (k = 0; k < t.length-1; k++)
        writer.write(this.getItemName(t[k]));
      writer.writeLast(this.getItemName(t[k]));
    }                           /* write items of the transaction */
  }  /* write() */

  /*------------------------------------------------------------------*/
  /** Parse a transaction.
   *  @param  iset   the underlying item set
   *  @param  reader the table reader to read from
   *  @return the parsed transaction
   *  @throws IOException if a read error occurs
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static int[] parseTrAct (IdMap iset, TableReader reader)
    throws IOException
  {                             /* --- parse a transaction */
    int   cnt = 0, n;           /* current number of items */
    int[] tract, t;             /* parsed transaction */

    tract = new int[BLKSIZE];   /* create an initial transaction */
    do {                        /* read the column names */
      if (reader.nextField() < 0)   /* if at the end of the input, */
        return null;            /* there is no transaction to read */
      /* Note that nextField() can return a negative value only   */
      /* on the first call, since a value of 0 (field separator)  */
      /* cannot be followed immediately by a negative value and   */
      /* a positive value (record separator) terminates the loop. */
      if (cnt >= tract.length){ /* if the transaction is full */
        n = cnt +((cnt > BLKSIZE) ? cnt >> 1 : cnt);
        System.arraycopy(tract, 0, t = new int[n], 0, cnt);
        tract = t;              /* create a larger transaction */
      }                         /* and copy the old one into it */
      tract[cnt] = iset.add(reader.field);
      cnt++;                    /* store and count the item */
    } while (reader.delim == 0);/* while in the same record */
    if (cnt < tract.length) {   /* if the transaction is not full */
      System.arraycopy(tract, 0, t = new int[cnt], 0, cnt);
      tract = t;                /* create a fitting transaction and */
    }                           /* copy the filled part into it */
    return tract;               /* return the created transaction */
  }  /* parseTrAct() */

  /*------------------------------------------------------------------*/
  /** Parse a set of transactions.
   *  @param  iset   the underlying item set
   *  @param  reader the table reader to read from
   *  @return the parsed set of transactions
   *  @throws IOException if a read error occurs
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static TrActSet parseTrActSet (IdMap iset, TableReader reader)
    throws IOException
  {                             /* --- parse a transaction set */
    TrActSet taset;             /* created transaction set */
    int[]    t;                 /* to traverse the transactions */

    taset = new TrActSet(iset); /* create a transaction set */
    while (true) {              /* transaction read loop */
      t = TrActSet.parseTrAct(taset.iset, reader);
      if (t == null) break;     /* read the next transaction */
      taset.addTrAct(t);        /* and add it to the set */
    }
    return taset;               /* return the created set */
  }  /* parseTrActSet() */

  /*------------------------------------------------------------------*/
  /** Main function for testing some basic functionality.
   *  <p>It is tried to parse the file that is given as the first
   *  command line argument as a set of transactions.</p>
   *  @param  args the command line arguments
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static void main (String args[])
  {                             /* --- main function for testing */
    TableReader reader;         /* table reader to read from */
    TrActSet    taset;          /* created transaction set */

    try {                       /* try to read the given file */
      if (args.length <= 0)     /* if no arguments given */
        reader = new TableReader("a b c\na d e f\nb d f\nc d e");
      else                      /* if a file argument is given */
        reader = new TableReader(new FileReader(args[0]));
      taset = TrActSet.parseTrActSet(null, reader);
      reader.close();           /* parse and print the trans. set */
      taset.write(new TableWriter(System.out)); }
    catch (IOException e) {     /* catch and report i/o errors */
      System.err.println(e.getMessage()); return; }
  }  /* main() */

}  /* class TrActSet */
