/*----------------------------------------------------------------------
  File    : TrActView.java
  Contents: transactions visualization program
  Author  : Christian Borgelt
  History : 2007.06.06 file created
            2007.06.07 loading and saving simplified
----------------------------------------------------------------------*/
package arules;

import java.io.IOException;
import java.io.File;
import java.awt.Component;
import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import util.TableReader;
import util.TableWriter;
import dialog.FormatDialog;
import dialog.AboutDialog;

/*--------------------------------------------------------------------*/
/** Class for a transaction viewer.
 *  @author Christian Borgelt
 *  @since  2007.06.06 */
/*--------------------------------------------------------------------*/
public class TrActView extends JFrame implements Runnable {

  private static final long serialVersionUID = 1L;

  /*------------------------------------------------------------------*/
  /*  constants                                                       */
  /*------------------------------------------------------------------*/
  /** mode flag: the viewer is a stand-alone program */
  public final static int PROGRAM    = 1;
  /** mode flag: add menu items for loading trees */
  public final static int LOAD_ITEMS = 2;
  /** mode flag: add menu items for saving trees */
  public final static int SAVE_ITEMS = 4;
  /** mode flag: add menu items for loading and saving trees */
  public final static int FILE_ITEMS = LOAD_ITEMS | SAVE_ITEMS;
  /** mode flag: add all menu items */
  public final static int ALL_ITEMS  = FILE_ITEMS;

  /*------------------------------------------------------------------*/
  /*  instance variables                                              */
  /*------------------------------------------------------------------*/
  /** the owner of this transactions viewer */
  private Component    owner   = null;
  /** the mode flags */
  private int          mode    = 0;
  /** the transactions table */
  private TrActTable   table   = null;
  /** the scroll pane for the viewport */
  private JScrollPane  scroll  = null;
  /** the table view */
  private JTable       view    = null;
  /** the status bar for messages */
  private JTextField   stat    = null;
  /** the file chooser */
  private JFileChooser chooser = null;
  /** the current transactions file */
  private File         curr    = null;
  /** the data format dialog box */
  private FormatDialog format  = null;
  /** the "About..." dialog box */
  private AboutDialog  about   = null;

  /*------------------------------------------------------------------*/
  /** Create a transactions viewer.
   *  @param  mode the mode flags
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TrActView (int mode)
  { this(null, mode); }

  /*------------------------------------------------------------------*/
  /** Create a transactions viewer.
   *  @param  owner the component that is to own this viewer
   *  @param  mode  the mode flags
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TrActView (Component owner, int mode)
  {                             /* --- create a transcations viewer */
    this.owner = null;          /* clear the owner and */
    this.mode  = mode;          /* note the mode flags */
    if (EventQueue.isDispatchThread()) { this.run(); return; }
    try { EventQueue.invokeAndWait(this); }
    catch (Exception e) {}      /* create the user interface */
  }  /* TrActView() */

  /*------------------------------------------------------------------*/
  /** Create the user interface.
   *  <p>Following the recommendations in the Java tutorial, the user
   *  interface is created in the "run" method, which is invoked from
   *  the event queue, in order to avoid problems with threads.</p>
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void run ()
  {                             /* --- create the user interface */
    Container content;          /* content pane of this frame */
    JMenuBar  mbar;             /* menu bar */
    JMenu     menu;             /* to create menu titles */
    JMenuItem item;             /* to create menu items */

    if ((this.mode & PROGRAM) != 0)
      this.mode |= LOAD_ITEMS;  /* check and adapt the mode */

    /* --- create and set the menu bar --- */
    this.setJMenuBar(mbar = new JMenuBar());

    menu = mbar.add(new JMenu("File"));
    menu.setMnemonic('f');
    if ((this.mode & LOAD_ITEMS) != 0) {
      item = menu.add(new JMenuItem("Load Transactions...", 'l'));
      item.addActionListener(new ActionListener() {
        public void actionPerformed (ActionEvent e) {
          TrActView.this.loadTrActs(null); } } );
      item = menu.add(new JMenuItem("Reload Transactions", 'r'));
      item.addActionListener(new ActionListener() {
        public void actionPerformed (ActionEvent e) {
          TrActView.this.loadTrActs(TrActView.this.curr); } } );
      menu.addSeparator();
    }
    if ((this.mode & SAVE_ITEMS) != 0) {
      item = menu.add(new JMenuItem("Save Transactions", 's'));
      item.addActionListener(new ActionListener() {
        public void actionPerformed (ActionEvent e) {
          TrActView.this.saveTrActs(TrActView.this.curr); } } );
      item = menu.add(new JMenuItem("Save Transactions as...", 'a'));
      item.addActionListener(new ActionListener() {
        public void actionPerformed (ActionEvent e) {
          TrActView.this.saveTrActs(null); } } );
      menu.addSeparator();
    }
    if ((this.mode & FILE_ITEMS) != 0) {
      item = menu.add(new JMenuItem("Data Format...", 'f'));
      item.addActionListener(new ActionListener() {
        public void actionPerformed (ActionEvent e) {
          TrActView.this.getFormatDlg().setVisible(true);
          TrActView.this.format.toFront();
        } } );
      menu.addSeparator();
    }

    item = menu.add(new JMenuItem("Quit", 'q'));
    if ((this.mode & PROGRAM) != 0) { /* if stand-alone program */
      item.addActionListener(new ActionListener() {
        public void actionPerformed (ActionEvent e) {
          System.exit(0); } } ); }     /* terminate the program */
    else {                      /* if only visualization module */
      item.addActionListener(new ActionListener() {
        public void actionPerformed (ActionEvent e) {
          TrActView.this.setVisible(false); } } );
    }                           /* close the window */

    menu = mbar.add(new JMenu("Help"));
    menu.setMnemonic('h');
    item = menu.add(new JMenuItem("About...", 'a'));
    item.addActionListener(new ActionListener() {
      public void actionPerformed (ActionEvent e) {
        if (TrActView.this.about == null)
          TrActView.this.about = new AboutDialog(TrActView.this,
             "About TrActView...", "TrActView\n"
            +"A Simple Transactions Viewer\n"
            +"Version 1.0, 2007.06.07\n\n"
            +"written by Christian Borgelt\n"
            +"European Center for Soft Computing\n"
            +"c/ Gonzalo Gutierrez Quiros s/n\n"
            +"33600 Mieres, Asturias, Spain\n"
            +"christian.borgelt@softcomputing.es");
        TrActView.this.about.setVisible(true);
        TrActView.this.about.toFront();
      } } );

    /* --- create and set the main panel --- */
    this.view   = new JTable();
    this.scroll = new JScrollPane(this.view);
    content = this.getContentPane();
    content.setLayout(new BorderLayout());
    content.add(this.scroll, BorderLayout.CENTER);

    /* --- create and set a status bar --- */
    stat = new JTextField("");
    stat.setEditable(false);
    content.add(this.stat,   BorderLayout.SOUTH);

    /* --- show the frame window --- */
    this.setTitle("TrActView");
    this.setDefaultCloseOperation(((this.mode & PROGRAM) != 0)
      ? JFrame.EXIT_ON_CLOSE : JFrame.HIDE_ON_CLOSE);
    if (this.owner == null) this.setLocation(48, 48);
    else                    this.setLocationRelativeTo(this.owner);
    this.pack();                /* configure the frame */
  }  /* run() */

  /*------------------------------------------------------------------*/
  /** Get the file chooser (create if necessary).
   *  @return the file chooser
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  private JFileChooser getChooser ()
  {                             /* --- get the file chooser */
    if (this.chooser != null)   /* if the chooser already exists, */
      return this.chooser;      /* simply return it */
    this.chooser = new JFileChooser();
    this.chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    this.chooser.setCurrentDirectory(new File("."));
    this.chooser.setFileHidingEnabled(true);
    this.chooser.setAcceptAllFileFilterUsed(true);
    this.chooser.setMultiSelectionEnabled(false);
    this.chooser.setFileView(null);
    return this.chooser;        /* create and configure file chooser */
  }  /* getChooser() */

  /*------------------------------------------------------------------*/
  /** Get the data format dialog (create if necessary).
   *  @return the data format dialog
   *  @since  2007.02.12 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public FormatDialog getFormatDlg()
  {                             /* --- get the data format dialog */
    if (this.format == null)    /* if it does not exist, create it */
      this.format = new FormatDialog(this, "Set Data Format...", 0);
    return this.format;         /* return the data format dialog */
  }  /* getFormatDlg() */

  /*------------------------------------------------------------------*/
  /** Set the message to display in the status line.
   *  @param  msg the message to display
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void setMessage (String msg)
  { this.stat.setText(msg); }

  /*------------------------------------------------------------------*/
  /** Set the transactions to display.
   *  @param  taset the transaction set to display
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void setTrActs (TrActSet taset)
  {                             /* --- set transactions to display */
    this.view.setModel(this.table = new TrActTable(taset));
    Dimension size = this.view.getPreferredSize();
    if (size.width  > 400) size.width  = 400;
    if (size.height > 400) size.height = 400;
    this.view.setPreferredScrollableViewportSize(size);
    this.pack();                /* re-layout the window */
  }  /* setTrActs() */

  /*------------------------------------------------------------------*/
  /** Get the currently displayed set of transactions.
   *  @return the currently displayed set of transactions
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TrActSet getTrActs ()
  { return this.table.getTrActs(); }

  /*------------------------------------------------------------------*/
  /** Report an I/O error.
   *  @param  msg the error message to report
   *  @since  2007.06.07 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  private void reportError (String msg)
  {                             /* --- report an i/o error */
    this.stat.setText(msg);     /* set message in status bar, */
    System.err.println();       /* print message to stderr, */
    System.err.println(msg);    /* and show alert dialog box */    
    JOptionPane.showMessageDialog(this,
      msg, "Error", JOptionPane.ERROR_MESSAGE);
  }  /* reportError() */

  /*------------------------------------------------------------------*/
  /** Load a set of transactions.
   *  @param  file the file to load the transactions from
   *  @return whether the file was successfully loaded
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public boolean loadTrActs (File file)
  {                             /* --- load a set of transactions */
    if (file == null) {         /* if no file name is given */
      int r = this.getChooser().showOpenDialog(this);
      if (r != JFileChooser.APPROVE_OPTION) return false;
      file = this.chooser.getSelectedFile();
    }                           /* get a selected file */
    try {                       /* create a reader for the file */
      System.err.print("reading " +file +" ... ");
      TableReader reader = this.getFormatDlg().createReader(file);
      TrActSet    taset  = TrActSet.parseTrActSet(null, reader);
      reader.close();           /* read the transactions and */
      this.setTrActs(taset);    /* set them for display */
      System.err.print("[" +taset.getTrActCount());
      System.err.println(" transaction(s)] done."); }
    catch (IOException e) {     /* catch and report an i/o error */
      this.reportError(e.getMessage()); return false; }
    this.stat.setText(file.getPath());
    this.curr = file;           /* note the new file name */
    return true;                /* return 'loading successful' */
  }  /* loadTrActs() */

  /*------------------------------------------------------------------*/
  /** Save the displayed set of transactions.
   *  @param  file the file to save the transactions to
   *  @return whether the file was successfully saved
   *  @since  2007.06.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public boolean saveTrActs (File file)
  {                             /* --- save displayed transactions */
    if (file == null) {         /* if no file name is given */
      int r = this.getChooser().showSaveDialog(this);
      if (r != JFileChooser.APPROVE_OPTION) return false;
      file = this.chooser.getSelectedFile();
    }                           /* get a selected file */
    try {                       /* create a writer for the file */
      System.err.print("writing " +file +" ... ");
      TableWriter writer = this.getFormatDlg().createWriter(file);
      TrActSet    taset  = this.table.getTrActs();
      taset.write(writer);      /* save the transactions */
      writer.close();           /* then close the file */
      System.err.print("[" +taset.getTrActCount());
      System.err.println("transaction(s)] done."); }
    catch (IOException e) {     /* catch and report an i/o error */
      this.reportError(e.getMessage()); return false; }
    this.curr = file;           /* note the new file name */
    return true;                /* return 'saving successful' */
  }  /* saveTrActs() */

  /*------------------------------------------------------------------*/
  /** Main function for command line invocation.
   *  @param  args the command line arguments
   *  @since  2004.07.06 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static void main (String args[])
  {                             /* --- main function */
    TrActView tv = new TrActView(PROGRAM|ALL_ITEMS);
    if (args.length > 0) tv.loadTrActs(new File(args[0]));
    tv.setVisible(true);        /* create and show transaction viewer */
  }  /* main() */

}  /* class TrActView */
